#!/usr/bin/env python

# Copyright (C) 2017 Alex Manuskin, Maor Veitsman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
from __future__ import absolute_import

from s_tui.ComplexBarGraphs import LabeledBarGraph
from s_tui.ComplexBarGraphs import ScalableBarGraph
import logging
logger = logging.getLogger(__name__)

class StuiBarGraph(LabeledBarGraph):

    @staticmethod
    def append_latest_value(values, new_val):

        values.append(new_val)
        return values[1:]

    MAX_SAMPLES = 1000
    SCALE_DENSITY = 5

    def __init__(self, source, color_a, color_b, smooth_a, smooth_b, alert_colors=None, bar_width = 1):
        self.source = source
        self.graph_name = self.source.get_source_name()
        self.measurement_unit = self.source.get_measurement_unit()

        self.num_samples = self.MAX_SAMPLES
        self.graph_data = [0] * self.num_samples

        self.color_a = color_a
        self.color_b = color_b
        self.smooth_a = smooth_a
        self.smooth_b = smooth_b

        self.alert_colors = alert_colors
        self.regular_colors = [color_a, color_b, smooth_a, smooth_b]

        self.satt = None


        x_label = []
        y_label = []

        w = ScalableBarGraph(['bg background', color_a, color_b])
        super(StuiBarGraph, self).__init__([w, x_label, y_label, self.graph_name + ' [' + self.measurement_unit + ']'])
        self.bar_graph.set_bar_width(bar_width)


    def get_current_summary(self):
        pass

    def get_graph_name(self):
        return self.graph_name

    def get_measurement_unit(self):
        return self.measurement_unit

    def get_is_available(self):
        return self.source.get_is_available()

    def get_label_scale(self, min_val, max_val, size):
        """Dynamically change the scale of the graph (y lable)"""
        if size < self.SCALE_DENSITY:
            label_cnt = 1
        else:
            label_cnt = int(size / self.SCALE_DENSITY)
        try:
            if max_val >= 100:
                label = [int((min_val + i * (max_val - min_val) / label_cnt))
                         for i in range(label_cnt + 1)]
            else:
                label = [round((min_val + i * (max_val - min_val) / label_cnt),1)
                         for i in range(label_cnt + 1)]
            return label
        except ZeroDivisionError:
            logging.debug("Side lable creation divided by 0")
            return ""

    def set_smooth_colors(self, smooth):
        if smooth:
            self.satt = {(1, 0): self.smooth_a, (2, 0): self.smooth_b}
        else:
            self.satt = None
        self.bar_graph.set_segment_attributes(['bg background', self.color_a, self.color_b], satt=self.satt)

    def set_regular_colors(self):
        self.color_a = self.regular_colors[0]
        self.color_b = self.regular_colors[1]
        self.smooth_a = self.regular_colors[2]
        self.smooth_b = self.regular_colors[3]
        if self.satt:
            self.satt = {(1, 0): self.smooth_a, (2, 0): self.smooth_b}
        self.bar_graph.set_segment_attributes(['bg background', self.color_a, self.color_b], satt=self.satt)

    def set_alert_colors(self):
        self.color_a = self.alert_colors[0]
        self.color_b = self.alert_colors[1]
        self.smooth_a = self.alert_colors[2]
        self.smooth_b = self.alert_colors[3]
        if self.satt:
            self.satt = {(1, 0): self.smooth_a, (2, 0): self.smooth_b}
        self.bar_graph.set_segment_attributes(['bg background', self.color_a, self.color_b], satt=self.satt)

    def update_displayed_graph_data(self):
        if not self.get_is_available():
            return

        # NOTE setting edge trigger causes overhead
        try:
            if self.source.get_edge_triggered():
                self.set_alert_colors()
            else:
                self.set_regular_colors()
        except NotImplementedError:
            pass


        l = []

        current_reading = self.source.get_reading()
        logging.info("Reading " + str(current_reading))
        data_max = self.source.get_maximum()
        self.graph_data = self.append_latest_value(self.graph_data, current_reading)

        # Get the graph width (dimension 1)
        num_displayed_bars = self.bar_graph.get_size()[1]
        # print num_displayed_bars
        # Iterage over all the information in the graph
        for n in range(self.MAX_SAMPLES-num_displayed_bars,self.MAX_SAMPLES):
            value = round(self.graph_data[n],1)
            # toggle between two bar types
            if n & 1:
                l.append([0, value])
            else:
                l.append([value, 0])

        self.bar_graph.set_data(l, float(data_max))
        y_label_size = self.bar_graph.get_size()[0]
        s = self.get_label_scale(0, float(data_max), float(y_label_size))
        self.set_y_label(s)

    def reset(self):
        self.graph_data = [0] * self.num_samples


    def get_summary(self):
        pass



#!/usr/bin/env python

# Copyright (C) 2017 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

"""ScalableBarGraph Class extends urwid.BarGraph so that
the current size of the bar graph is also obtainable
get_size() - returns the tuple (row, col)
"""

from __future__ import absolute_import

import urwid
#from graphics import BarGraph


class ScalableBarGraph(urwid.BarGraph):
    """Scale the graph acording to screen size"""
    _size = (0, 0)

    def render(self, size, focus=False):
        canvas = super(ScalableBarGraph, self).render(size, focus)
        new_size = (int(canvas.rows()), int(canvas.cols()))
        old_size = self._size
        # check if to raise *on_resize* event
        if new_size != old_size:
            self.on_resize(new_size)
        self._size = new_size
        return canvas

    def calculate_bar_widths(self, size, bardata):
        """
        Return a list of bar widths, one for each bar in data.

        If self.bar_width is None this implementation will stretch
        the bars across the available space specified by maxcol.
        """
        (maxcol, maxrow) = size

        if self.bar_width is not None:
            return [self.bar_width] * min(
                len(bardata), int(maxcol / self.bar_width))

        if len(bardata) >= maxcol:
            return [1] * maxcol

        widths = []
        grow = maxcol
        remain = len(bardata)
        for row in bardata:
            w = int(float(grow) / remain + 0.5)
            widths.append(w)
            grow -= w
            remain -= 1
        return widths


    def get_size(self):
        return self._size

    def on_resize(self, new_size):
        pass  # place folder for any future implantation


class LabeledBarGraph(urwid.Pile):
    """Add option to add lables for X and Y axes """
    def __init__(self, widget_list, focus_item=None):
        if len(widget_list) != 4:
            raise Exception(
                'Must have 3 items for labeled bar graph')
        if not isinstance(widget_list[0], ScalableBarGraph):
            raise Exception(
                'Item 0 must be ScalableBarGraph')
        if not self.check_label(widget_list[1]):
            raise Exception(
                'Item 1 must be a valid label')
        if not self.check_label(widget_list[2]):
            raise Exception(
                'Item 2 must be a valid label')

        self.bar_graph = []
        self.set_graph(widget_list[0])

        self.y_label = urwid.WidgetPlaceholder(urwid.Columns([]))
        self.set_y_label(widget_list[2])

        self.x_label = urwid.WidgetPlaceholder(urwid.Columns([]))
        self.set_x_label(widget_list[1])

        self.title = urwid.WidgetPlaceholder(urwid.ListBox([]))
        self.set_title(widget_list[3])

        super(LabeledBarGraph, self).__init__([
            ('fixed', 0 if len(widget_list[3]) == 0 else 1, self.title),
            self.y_label,
            ('fixed', 0 if len(widget_list[1]) == 0 else 1, self.x_label)
        ], focus_item=focus_item)
        # if self.y_label is not None and self.x_label is not None:
        #     self.y_label = ('weight', 1, self.y_label)
        #
        # elif self.y_label is not None:
        #     super(LabeledBarGraph, self).__init__([self.title, self.y_label], focus_item=focus_item)
        # elif self.x_label is not None:
        #     super(LabeledBarGraph, self).__init__([self.title, self.bar_graph, self.x_label], focus_item=focus_item)
        # else:
        #     super(LabeledBarGraph, self).__init__([self.title, self.bar_graph], focus_item=focus_item)

    def set_title(self, title):
        if len(title) == 0:
            return

        self.title.original_widget = urwid.ListBox([urwid.Text(title, align="center")])

    def set_x_label(self, x_label):
        if len(x_label) == 0:
            return

        str_x_label = [str(i) for i in x_label]
        x_label_nums = str_x_label[1:]

        x_label_num_list = [urwid.ListBox([urwid.Text('  ' + str_x_label[0])])]

        for num in x_label_nums:
            x_label_num_list = x_label_num_list + [urwid.ListBox([urwid.Text(num)])]
        x_label_num_list[-1] = (1, x_label_num_list[-1])

        self.x_label.original_widget = urwid.Columns(x_label_num_list)

    def set_y_label(self, y_label):
        if len(y_label) == 0:
            return

        str_y_label = [str(i) for i in y_label]
        y_label_nums = str_y_label[1:]
        y_list_walker = [(1, urwid.ListBox([urwid.Text(str_y_label[0])]))]

        for num in y_label_nums:
            y_list_walker = [urwid.ListBox([urwid.Text(num)])] + y_list_walker

        y_list_walker = urwid.Pile(y_list_walker, focus_item=0)
        y_scale_len = len(max(str_y_label, key=len))

        y_notation = [('fixed',  y_scale_len,        y_list_walker),
                      ('weight', 1,                  self.bar_graph)]

        self.y_label.original_widget = urwid.Columns(y_notation,
                                                     dividechars=1)

    def set_graph(self, graph):
        self.bar_graph = graph

    @staticmethod
    def check_label(label):
        if len(label) >= 2 and not(None in label) or len(label) == 0 or label is None:
            return True

        return False



#!/usr/bin/env python

# Copyright (C) 2017 Alex Manuskin, Maor Veitsman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
from __future__ import absolute_import

import time
import psutil
from s_tui.Sources.Source import Source

import logging
logger = logging.getLogger(__name__)


class UtilSource(Source):

    def __init__(self):
        self.last_freq = 0
        self.update()

    def update(self):
        result = 0
        try:
            result = float(psutil.cpu_percent(interval=0.0))
        except:
            result = 0
            logging.debug("Cpu Utilization unavailable")

        self.last_freq = float(result)
        logging.info("Utilization recorded " + str(self.last_freq))

    def get_reading(self):
        return self.last_freq

    def get_maximum(self):
        return 100

    def get_is_available(self):
        return True

    def get_summary(self):
        return {'Utilization': '%.1f %s' % (self.last_freq, self.get_measurement_unit())}

    def get_source_name(self):
        return 'Utilization'

    def get_measurement_unit(self):
        return '%'



if '__main__' == __name__:
    util = UtilSource()
    while True:
        print(util.get_reading())
        time.sleep(2)



#!/usr/bin/env python

# Copyright (C) 2017 Alex Manuskin, Maor Veitsman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
from __future__ import absolute_import

import os
import time
import math

from s_tui.Sources.Source import Source

import logging
logger = logging.getLogger(__name__)

class RaplPowerSource(Source):

    intel_rapl_folder = '/sys/class/powercap/intel-rapl/'

    MICRO_JOULE_IN_JOULE = 1000000.0

    def __init__(self, package_number = 0):
        self.package_number = package_number
        self.intel_rapl_package_energy_file = os.path.join(self.intel_rapl_folder, 'intel-rapl:%d'%package_number, 'energy_uj')
        self.intel_rapl_package_max_energy_file = os.path.join(self.intel_rapl_folder, 'intel-rapl:%d'%package_number, 'constraint_0_max_power_uw')
        if (not os.path.exists(self.intel_rapl_package_energy_file) or not os.path.exists(self.intel_rapl_package_max_energy_file)):
            self.is_available = False
            self.last_measurement_time = 0
            self.last_measurement_value = 0
            self.max_power = 0
            self.last_watts = 0
            return

        self.is_available = True
        self.last_measurement_time = time.time()
        self.last_measurement_value = self.read_power_measurement_file()
        #self.max_power = self.read_max_power_file() / self.MICRO_JOULE_IN_JOULE
        self.max_power = 1
        self.last_watts = 0

        self.update()

    def read_measurement(self, file_path):
        try:
            with open(file_path) as f:
                value = f.read()
                return float(value)
        except:
            return 0

    def read_max_power_file(self):
        if not self.is_available:
            return -1
        return float(self.read_measurement(self.intel_rapl_package_max_energy_file))

    def read_power_measurement_file(self):
        if not self.is_available:
            return -1
        return float(self.read_measurement(self.intel_rapl_package_energy_file))

    def get_power_usage(self):
        if not self.is_available:
            return -1
        current_measurement_value = self.read_power_measurement_file()
        current_measurement_time = time.time()

        joule_used = (current_measurement_value - self.last_measurement_value) / float(self.MICRO_JOULE_IN_JOULE)
        logging.info("current " + str(current_measurement_value) + " last " + str(self.last_measurement_value))
        seconds_passed = current_measurement_time - self.last_measurement_time
        watts_used = joule_used / seconds_passed
        logging.info("Joule_Used " + str(joule_used) + " seconds_passed " + str(seconds_passed))

        self.last_measurement_value = current_measurement_value
        self.last_measurement_time = current_measurement_time
        self.last_watts = watts_used
        try:
            if watts_used > self.max_power:
                self.max_power = math.ceil(watts_used)
                logging.info("Max power updated " + str(self.max_power))
        except:
            self.max_power = 1
        return watts_used

    # Source super class implementation
    def get_is_available(self):
        return self.is_available

    def update(self):
        self.get_power_usage()

    def get_reading(self):
        return self.last_watts

    def get_maximum(self):
        return self.max_power

    def reset(self):
        self.max_power = 1
        self.last_watts = 0

    def get_summary(self):
        return {'Cur Power': '%.1f %s' % (self.last_watts, self.get_measurement_unit())
                , 'Max Power': '%.1f %s' % (self.max_power, self.get_measurement_unit())}

    def get_source_name(self):
        return 'Power'

    def get_measurement_unit(self):
        return 'W'


if '__main__' == __name__:
    rapl = RaplPower()
    while True:
        print(rapl.get_power_usage())
        time.sleep(2)



#!/usr/bin/env python
#
# Copyright (C) 2017 Alex Manuskin, Gil Tsuker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.


""" Reads the value of the msr containing information on Turbo Boost on intel CPUs
"""
import os
import logging
import signal
import platform
import subprocess
import re
import csv
import sys
import json
import time

from collections import OrderedDict
from sys import exit

__version__ = "0.6.4"


def get_processor_name():
    if platform.system() == "Windows":
        return platform.processor()
    elif platform.system() == "Darwin":
         return subprocess.check_output(['/usr/sbin/sysctl', "-n", "machdep.cpu.brand_string"]).strip()
    elif platform.system() == "Linux":
        command = "cat /proc/cpuinfo"
        all_info = subprocess.check_output(command, shell=True).strip()
        for line in all_info.split(b'\n'):
            if b'model name' in line:
                return re.sub( b'.*model name.*:', b'', line,1)
    return ""

def kill_child_processes(parent_proc, sig=signal.SIGTERM):
    try:
        for proc in parent_proc.children(recursive=True):
            logging.debug('Killing' + str(proc))
            proc.kill()
        parent_proc.kill()
    except:
        logging.debug('No such process')

def output_to_csv(sources, csv_writeable_file):
    """Print statistics to csv file"""
    file_exists = os.path.isfile(csv_writeable_file)

    with open(csv_writeable_file, 'a') as csvfile:
        csv_dict = OrderedDict()
        csv_dict.update({'Time': time.strftime("%Y-%m-%d_%H:%M:%S")})
        summaries = [val for key,val in sources.items()]
        for summarie in summaries:
            csv_dict.update(summarie.source.get_summary())


        fieldnames = [key for key,val in csv_dict.items()]
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)

        if not file_exists:
            writer.writeheader()  # file doesn't exist yet, write a header
        writer.writerow(csv_dict)

def output_to_terminal(sources):
    """Print statistics to the terminal"""
    results = OrderedDict()
    for s in sources:
        if s.get_is_available():
            s.update()
    for s in sources:
        if s.get_is_available():
            results.update(s.get_summary())
    for key,value in  results.items():
        sys.stdout.write(str(key) + ": " + str(value) + ", ")
    sys.stdout.write("\n")
    exit()

def output_to_json(sources):
    """Print statistics to the terminal"""
    results = OrderedDict()
    for s in sources:
        if s.get_is_available():
            s.update()
    for s in sources:
        if s.get_is_available():
            results.update(s.get_summary())
    print(json.dumps(results, indent=4))
    exit()


DEFAULT_PALETTE = [
    ('body',                    'default',        'default',   'standout'),
    ('header',                  'default',        'dark red',     ),
    ('screen edge',             'light blue',     'brown'),
    ('main shadow',             'dark gray',      'black'),
    ('line',                    'default',          'light gray',   'standout'),
    ('menu button',             'light gray',     'black'),
    ('bg background',           'default',         'default'),
    ('overheat dark',           'white',          'light red',     'standout'),
    ('bold text',               'default,bold',          'default',     'bold'),
    ('under text',               'default,underline',          'default',     'underline'),

    ('util light',              'default',        'light green'),
    ('util light smooth',       'light green',     'default'),
    ('util dark',               'default',       'dark green'),
    ('util dark smooth',        'dark green',    'default'),

    ('high temp dark',          'default',       'dark red'),
    ('high temp dark smooth',   'dark red',      'default'),
    ('high temp light',         'default',       'light red'),
    ('high temp light smooth',  'light red',     'default'),

    ('power dark',               'default',      'light gray', 'standout'),
    ('power dark smooth',        'light gray',   'default'),
    ('power light',              'default',      'white', 'standout'),
    ('power light smooth',       'white',    'default'),

    ('temp dark',               'default',        'dark cyan',    'standout'),
    ('temp dark smooth',        'dark cyan',      'default'),
    ('temp light',              'default',       'light cyan',   'standout'),
    ('temp light smooth',       'light cyan',     'default'),

    ('freq dark',               'default',        'dark magenta', 'standout'),
    ('freq dark smooth',        'dark magenta',   'default'),
    ('freq light',              'default',        'light magenta', 'standout'),
    ('freq light smooth',       'light magenta',  'default'),

    ('button normal',           'dark green',     'default',    'standout'),
    ('button select',           'white',          'dark green'),
    ('line',                    'default',        'default',      'standout'),
    ('pg normal',               'white',          'default',        'standout'),
    ('pg complete',             'white',          'dark magenta'),
    ('high temp txt',           'light red',      'default'),
    ('pg smooth',               'dark magenta',   'default')
    ]

if '__main__' == __name__:
    avg = get_avarage_cpu_freq()
    print(avg)



import psutil
import os
import re
import subprocess

import logging
logger = logging.getLogger(__name__)

class GlobalData:

    def __init__(self, is_admin):
        self.is_admin = is_admin
        self.num_cpus = 1
        try:
            self.num_cpus = psutil.cpu_count()
            logging.info("num cpus " + str(self.num_cpus))
        except (IOError, OSError) as e:
            logging.debug(e.message)
